;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x360837c27617fc1baa5a6d26638a5b00 . "./code.cg2js/cg-status-bar.fasl")
(#x716da0cf727e135469528c7670a0f23e . "./code.cg2js/cg-hotspot.fasl")
(#xfe2111585e49c6a5a144713e1547a8d5 . "./code.cg2js/cg-toolbar.fasl")
(#x21c6ece24423c288f7776e52dffbc183 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xa6a8baeab652d81db599ea46667e2614 . "./code.cg2js/cg-tooltip.fasl")
(#xd4b795a3f404885ba7d455cb3819ecf7 . "./code.cg2js/cg-calendar.fasl")
(#x2741d12b2871c02f01aef27e392cf9d8 . "./code.cg2js/cg-item-list.fasl")
(#xcd73d3765343040c7a7271435030e13f . "./code.cg2js/cg-bitmap-stream.fasl")
(#xb6e7aa5893de313e986fcdefa10b632e . "./code.cg2js/cg-trackbar.fasl")
(#x1b829953fde18fa68cd887828e317efc . "./code.cg2js/cg-class-grid.fasl")
(#x9be0a92dcebe3e96ec306dad930abfb6 . "./code.cg2js/cg-dialog-item.fasl")
(#xbe01bb5789596f6e22a143b6acd20d9c . "./code.cg2js/cg-comtab.fasl")
(#xd6c2fd36e8ac70625ff37c282a161a22 . "./code.cg2js/cg-lisp-text.fasl")
(#xebbd3eaf0ff7c8a15af31f94cc56bbf8 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x52f89e8346c034c0611fc8db3f01e4d1 . "./code.cg2js/cg-button.fasl")
(#x4ae6a69fd7e03fb660184b1a2202ed2d . "./code.cg2js/cg-printing.fasl")
(#x0db124841e90b9e1e04ec53d9190edc6 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xfd9bab7244d448eaf9571474219cbf39 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x15b75c28b629e5bd159a6e07973e6922 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x200fb99b089502f867965bc7ad592f2c . "./code.cg2js/cg-selected-object.fasl")
(#x84339e3ca4530ecad11803b2a0476713 . "./code.cg2js/cg-text-or-combo.fasl")
(#x940f5f7e8663119421a67a5a60cc3c43 . "./code.cg2js/cg-graphics-context.fasl")
(#x92fa911d14d216ae5e47162f3ef9ccc7 . "./code.cg2js/cg-edit-in-place.fasl")
(#x376ed5c0d76d6d3db35edabba3f1d4d3 . "./code.cg2js/cg-string-dialog.fasl")
(#x80933fb66cac07a47ec8ead637ca3f7d . "./code.cg2js/cg-list-view.fasl")
(#xdcabdadf4a562b1c043ea0d1b4555790 . "./code.cg2js/cg-intersect.fasl")
(#xbb43c94437a957bcb172a47f0280107f . "./code.cg2js/cg-text-widget.fasl")
(#x58ef387d24e80be06cfc078abb65f9fe . "./code.cg2js/cg-directory-dialog.fasl")
(#x183f680913e0c0556c12947db27306cc . "./code.cg2js/cg-file-dialog.fasl")
(#x157221d881dd574c584ef51686518e2a . "./code.cg2js/cg-dde-utils.fasl")
(#x9a306f60c00a38be1bc1267bc10088a4 . "./code.cg2js/cg-nodes.fasl")
(#xb3cdc0361f95e84e0841eccbb305a744 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xd4d8c90e12bd21f78f42e309c799ae4a . "./code.cg2js/cg-up-down-control.fasl")
(#xe96d546e91dc802739d9102c3ca1dad5 . "./code.cg2js/cg-get-position.fasl")
(#x77688559e46283414fb67fbb7137bd38 . "./code.cg2js/cg-toggling-widget.fasl")
(#xdcb03f3b2c024b79fb9c31e01ef420f7 . "./code.cg2js/cg-acache.fasl")
(#xaa838ff85d028667360a7e801e9050a8 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xd891eefa9d0db16049c00640fb38031a . "./code.cg2js/cg-base.fasl")
(#xd122f3cc18ba2f1e8e1886698f8d8ff6 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x209811e6c9cca3bf44e3e213fc594088 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x9f5da0b102a2ca9fce0ae89214834420 . "./code.cg2js/cg-property.fasl")
(#x12ee02baceb1841e6bcaa2b5d2e7dcb4 . "./code.cg2js/cg-editable-text.fasl")
(#x6e7c65592cc40fd0856957470efa7b0f . "./code.cg2js/cg-lamp.fasl")
(#x26f0c7e0d6708ca261b0871a63b4bee9 . "./code.cg2js/cg-progress-indicator.fasl")
(#x92b88a201f5782a40f7d910407c39942 . "./code.cg2js/cg-chart-widget.fasl")
(#xd095b9cdd51246e3daa553059a4abfa2 . "./code.cg2js/cg-common-control.fasl")
(#x4de4e5b963a2c2f32284f12a6e258fa0 . "./code.cg2js/cg-font-dialog.fasl")
(#x9bb4f8fdc57bde05f8f101d1e370b948 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xd88be49d50277a196aaaa3479d632b16 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x83913fd1869f5d92ebc907df08efbb7b . "./code.cg2js/cg-os-window.fasl")
(#x9375be36a152365a9393d5038f372c7c . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x61bfdd1e2d96af69d1d0a45cdc168cbf . "./code.cg2js/cg-pixmap.fasl")
(#x9a59d53df38195716fe5a64eb912bcef . "./code.cg2js/cg-chart-or-plot.fasl")
(#xe7aadeafffb36de0b35b079b1cfe7b7b . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x5dbe01e121317832152d1717fea61ecb . "./code.cg2js/cg-scaling-stream.fasl")
(#x146e80ce73ee31a586fbcc0ef594ac58 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xd4ee847e64da44b400b66984931d49fa . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x7d755455936081a6345795fe3eaca2bc . "./code.cg2js/cg-timer.fasl")
(#x7c76fa77873a29f390b2ec7556eabd37 . "./code.cg2js/cg-scroll-bar.fasl")
(#x3fefb6ff42571c8d00ce8383fcd00f8d . "./code.cg2js/cg-caret.fasl")
(#xdd79241ee864a84df456cd31921786a3 . "./code.cg2js/cg-template-string.fasl")
(#x1a024d4960bf49f9fca98d85064fed10 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xea43cabe7463d08ac58f9938c2b2ef86 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xdc5712a1a2e53ececcbc125402cf1d08 . "./code.cg2js/cg-rich-edit.fasl")
(#xc018b379524efbd718bdb9bb91ed5423 . "./code.cg2js/cg-combo-box.fasl")
(#x51ea68c911183163df68b44c39a60f7a . "./code.cg2js/cg-text-edit-pane.fasl")
(#xbc89670f6c9b1093b90f2d56bb63bf18 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xe1dc98a18ade61a3af29329a4701f26e . "./code.cg2js/cg-menu-tooltip.fasl")
(#x0bf4aaf549cf64af4fe082589366fbea . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x06e8fa7aa02b28ee4240e080be641d20 . "./code.cg2js/cg-outline.fasl")
(#xbc7994c990c76dc04ccc830ae2727d8e . "./code.cg2js/cg-class-support.fasl")
(#x5e9bacb553d0d1d33bffbc1733599d5d . "./code.cg2js/cg-header-control.fasl")
(#x4b6e9b172b1b3482545fabdc9696d118 . "./code.cg2js/cg-lettered-menu.fasl")
(#x1449c68b33bcb628bc74e0c0c99342e1 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x49c6825dffa8429657bc1a7b3ee8c717 . "./code.cg2js/cg-fill-texture.fasl")
(#x5e438aeb2a04eda1f9af48a40521e673 . "./code.cg2js/cg-lisp-widget.fasl")
(#x5dd88d8859a731296078b82cc4904c41 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x56e389242fae194f382e23c6e82f0e17 . "./code.cg2js/cg-clipboard.fasl")
(#xbc6680c8490a012736e200bebe719283 . "./code.cg2js/cg-wrap-string.fasl")
(#x19f000bdf642e7bf6e3652a9381a2053 . "./code.cg2js/cg-os-widget.fasl")
(#xa24cd2b4bcd21cd2e06421cb161bc51b . "./code.cg2js/cg-check-box.fasl")
(#xb1a950475abdaeb4756d6824c7fc60a5 . "./code.cg2js/cg-choice-list.fasl")
(#xaaa23c4111370b12baa40e9bc7c99b5c . "./code.cg2js/cg-layout.fasl")
(#x8c0568c4b6f5f7e8cdfd84122e3ea67c . "./code.cg2js/cg-group-box.fasl")
(#xb09d563faee4fe2a72e8c46580f614c7 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x11c5fd4d2c77374af533346f06ef81ee . "./code.cg2js/cg-radio-button.fasl")
(#x4aa50dde47cd003a9ef15265b1a1f953 . "./code.cg2js/cg-tab-control.fasl")
(#x4a1c075d1ae79a989ca10ae3f14b3b29 . "./code.cg2js/cg.fasl")
(#xa2973adb60ab4f39d1b4e2d7ab286ec0 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xb7f9f7acb839b56fe91a19d03338098b . "./code.cg2js/cg-dropping-outline.fasl")
(#xde6de80a3b75c96d13233ea233885f9f . "./code.cg2js/cg-get-pixmap.fasl")
(#x5abaf31555fa383d423d24c475a3ce4a . "./code.cg2js/cg-grid-widget.fasl")
(#x61a66b077dcbbf56010a98e4467ad0c0 . "./code.cg2js/cg-utility-dialog.fasl")
(#xeda75f89fcd3da3f6785a27a4ee736e2 . "./code.cg2js/cg-static-text.fasl")
(#xabbbc2f8078fc819f7fdea4d89942f70 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x34fb41974075c1e4a1b3620bf166890e . "./code.cg2js/cg-object-editor-layout.fasl")
(#x1359a142d06f813e49d28f7f818a165d . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x900e2cdf7869406a9c25a7c49a877c02 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x9fbe9486e1ad6ad1aeb45ee1744be816 . "./code.cg2js/cg-plot-widget.fasl")
(#x27dacd5ae8fa5b207c4295a139fc5a9a . "./code.cg2js/cg-color-dialog.fasl")
(#x317b17bc310a995975688e93eeaa35dd . "./code.cg2js/cg-icon-pixmap.fasl")
(#xec1048bf43b89fe0d5b536b071bb78d1 . "./code.cg2js/cg-icon.fasl")
(#xd59431a285e994a1acc1624adb733186 . "./code.cg2js/cg-dev.fasl")
(#x1aca686d64f03dbc747edb56fda722c2 . "./code.cg2js/cg-choose-printer.fasl")
(#x818cd8eb3b763a14900fda29a1cf7129 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xb78d59178f925dfb3ded73792857efa1 . "./code.cg2js/cg-menu.fasl")
(#xc2ffe613701aeed67109070c30c2e920 . "./code.cg2js/cg-yes-no-list.fasl")
(#x50b5dd390a05884237a29b56185c7d4f . "./code.cg2js/cg-palette.fasl")
(#x5904c95485997b658e7ba126fa965050 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x260f5e321ad9287d203daa03dfaa0b40 . "./code.cg2js/cg-message-dialog.fasl")
(#xc6313ac953b597bf092b3d41a4688348 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xe4216f807751af2f763f238877a7280e . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x2c036f31559f4dbfd044ab2cb0c26419 . "./code.cg2js/cg-split-bar.fasl")
(#xd1e434124e188e2809e6c00721dcbd79 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x518b958e724471e007fc7134ed950c1e . "./code.cg2js/cg-web-browser.fasl")
(#x78b286accdae3f5db44534f3d42de5d4 . "./code.cg2js/cg-project-window.fasl")
(#xb126d389dc5caa2dcf3ea45878a08cfa . "./code.cg2js/cg-picture-widget.fasl")
(#x094c55fadf442d049b2f324247f0b7b1 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x775786d8333b42b1fb667881afa04379 . "./code.cg2js/cg-object-editor.fasl")
(#xc4e8cfcebdcee9ca2317c48345538574 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x305eacc3ec3b5a57dca539139120af36 . "./code.cg2js/cg-paren-matching.fasl")
(#xf6a8c78fa21c8a32b68eb60e77a33a90 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x8403210cf71f1dc103ab97e8fc720927 . "./code.cg2js/cg-drawable.fasl")
(#x0a74fa6aafec210d0272f3bf34b76fe9 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xb45730d5c9ee361cec46cd0d334cdacf . "./code.cg2js/cg-html-widget.fasl")
(#xa23e55d7eaea564c9d6e4fe43601aa69 . "./code.cg2js/cg-curve.fasl")
(#x45f218258743360e76dbc42a4ef335f8 . "./code.cg2js/cg-yes-no-string.fasl")
(#x34f46c6aff163bfa6f51bfb3868cf623 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x531b3c93e42cd872275f552afe28a7a1 . "./code.cg2js/cg-html-dialog.fasl")
(#x324a2a70bcf888f2864fb0a2ee2c45f8 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x89d2f1e173ebac8743853695109ba26a . "./code.cg2js/cg-tray.fasl")
(#x34919b2cf90547e43e16a5574f2b820a . "./code.idejs/ide-apropos.fasl")
(#x16ad15653a8829fa35aacabd5bd41c04 . "./code.idejs/ide-window-list.fasl")
(#x204729ee74883e7ce027db96a285a27d . "./code.idejs/ide-class-browser.fasl")
(#x8f0744cbe74984cb8b2f66a2ed11da6d . "./code.idejs/ide-base.fasl")
(#x3069ef32442c5d6eb112c9f99c3b58e7 . "./code.idejs/ide-process-browser.fasl")
(#x9ae7a69da777ff8f293e55868b640e8e . "./code.idejs/ide-acache.fasl")
(#x54eb10c575c10932cebe596ddb2df33e . "./code.idejs/ide-navigator.fasl")
(#x5a713f6e2954100b976931becef005e3 . "./code.idejs/ide-trace.fasl")
(#xdcce320c8ebdf452978ee3e667daf432 . "./code.idejs/ide-project.fasl")
(#x3a7fa55d1add1dde6dab6db9509d34dc . "./code.idejs/ide-patch-dialog.fasl")
(#x181a58664dc4e706144f35033397c7e7 . "./code.idejs/ide-grapher.fasl")
(#xfd4fd6092f4980d0f233db1703e94779 . "./code.idejs/ide-dev.fasl")
(#x93c96e8938cd9d6266a97d2d7570437c . "./code.idejs/ide-keyshort.fasl")
(#x70c4deedf334cbfa4a07b124b69eebb9 . "./code.idejs/ide-editor.fasl")
(#x82217ef7af7f4028b2fcb75fbf85e3bd . "./code.idejs/ide-inspector.fasl")
(#xa80efa7f9dcc725b4f5fb2caf0495a5c . "./code.idejs/ide.fasl")
(#x64c245594a0a10144e019408a42dd795 . "./code.idejs/ide-clipboard.fasl")
(#x1859ea0a06fafbb3a98f1791102184bb . "./code.idejs/ide-find-definitions.fasl")
(#xe40dec4e235f884e641f1214adc5052a . "./code.idejs/ide-form.fasl")
(#x66d09298821973c9605ed37abc74927e . "./code.idejs/ide-intro.fasl")
(#x42a9725e6a12005319054715a36c1b0f . "./code.idejs/ide-find-in-files.fasl")
(#xe9bb18052dc341463da4754823e0535c . "./code.idejs/ide-package-list.fasl")
(#xd08280202e8456f262b8c5cb9f284627 . "./code.idejs/ide-class-grapher.fasl")
(#xe396744af4d34e05e734175c54f8cafc . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
